package info.bukova.isspst.security;

import info.bukova.isspst.Constants;
import info.bukova.isspst.Module;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.services.Service;

import java.io.Serializable;
import java.util.List;

import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.core.Authentication;

public class IsspstPermissionEvaluator implements PermissionEvaluator {

	@SuppressWarnings("unchecked")
	@Override
	public boolean hasPermission(Authentication authentication,
			Object targetDomainObject, Object permission) {
		List<Role> perms = (List<Role>) authentication.getAuthorities();
		String perm = "";
		
		if (permission instanceof String) {
			perm = (String) permission;
		}
		
		if (targetDomainObject instanceof Service<?>)
		{
			return evaluateGlobal((Service<?>) targetDomainObject, perm, perms);
		}
		
		return false;
	}

	@Override
	public boolean hasPermission(Authentication authentication,
			Serializable targetId, String targetType, Object permission) {
		return false;
	}
	
	private boolean evaluateGlobal(Service<?> service, String permission, List<Role> perms) {
		
		String moduleId = "";
		String perm = permission;
		
		for (Module m : Constants.MODULES) {
			if (m.getServiceClass() != null && m.getServiceClass().isAssignableFrom(service.getClass())) {
				moduleId = m.getId();
				break;
			}
		}
		
		perm += "_" + moduleId;
		
		for (int i = 0; i < perms.size(); i++) {
			if (!(perms.get(i) instanceof Role)) {
				return false;
			}
			if (perms.get(i).getAuthority().equals(perm)) {
				return true;
			}
			if (perms.get(i).getAuthority().equals(Constants.ROLE_ADMIN)) {
				return true;
			}
		}
		
		return false;
	}

}
